/*------------------------------------------------------------------------------
 bin/djv_view/cache.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "cache.h"

namespace cache {

Ref::Ref(image::Image * in, int key, int frame) :
  _image(in), _key(key), _frame(frame),
  _ref_count(1)
{}

void Ref::key(int in)
{
  _key = in;
}

void Ref::ref_del()
{
  --_ref_count;
}

Cache::Cache() :
  _cache_max(256 * mem::megabyte), _cache_size(0)
{
  DJV_DBG_OBJ("cache::Cache::Cache");
}

Cache::~Cache()
{
  DJV_DBG_OBJ("cache::~Cache::Cache");
#if defined(DJV_DBG)
  dbg();
#endif
  
  for (uint i = 0; i < _ref_list.size(); ++i)
    delete _ref_list[i];
}

Ref * Cache::create(image::Image * image, int key, int frame)
{
  DJV_DBG_OBJ("cache::Cache::create");
  DJV_DBG_PRN("image = " << *image);
  DJV_DBG_PRN("key = " << key);
  DJV_DBG_PRN("frame = " << frame);
#if defined(DJV_DBG)
  dbg();
#endif

  Ref * out = new Ref(image, key, frame);
  _ref_list += out;
  _cache_size += out->get()->data_bytes();

  if (_cache_size > _cache_max) purge();

#if defined(DJV_DBG)
  dbg();
#endif
  return out;
}

Ref * Cache::get(int key, int frame)
{
  DJV_DBG_OBJ("cache::Cache::get");
  DJV_DBG_PRN("key = " << key);
  DJV_DBG_PRN("frame = " << frame);
#if defined(DJV_DBG)
  //dbg();
#endif

  Ref * out = 0;
  for (List<Ref *>::iterator i = _ref_list.begin(); i != _ref_list.end(); ++i)
    if (key == (*i)->key() && frame == (*i)->frame()) {
      out = *i;
      out->ref();
      _ref_list.erase(i);
      _ref_list += out;
      break;
    }

#if defined(DJV_DBG)
  dbg();
#endif

  DJV_DBG_PRN("out = " << (out ? out->key() : 0));
  return out;
}

void Cache::max(int in)
{
  DJV_DBG_OBJ("cache::Cache::max");
  DJV_DBG_PRN("in = " << in);
#if defined(DJV_DBG)
  dbg();
#endif
  
  _cache_max = in * mem::megabyte;
  
  if (_cache_size > _cache_max) purge();
  
#if defined(DJV_DBG)
  dbg();
#endif
}

int Cache::max() const
{
  return _cache_max / mem::megabyte;
}

void Cache::del(int key)
{
  DJV_DBG_OBJ("cache::Cache::del");
  DJV_DBG_PRN("key = " << key);
#if defined(DJV_DBG)
  dbg();
#endif

  for (uint i = 0; i < _ref_list.size(); ++i)
    if (key == _ref_list[i]->key())
      _ref_list[i]->key(0);

  purge();
  
#if defined(DJV_DBG)
  dbg();
#endif
}

void Cache::del()
{
  DJV_DBG_OBJ("cache::Cache::del");
#if defined(DJV_DBG)
  dbg();
#endif

  List<Ref *> tmp;
  
  for (uint i = 0; i < _ref_list.size(); ++i)
    if (! _ref_list[i]->ref_count()) {
      _cache_size -= _ref_list[i]->get()->data_bytes();
      delete _ref_list[i];
    } else
      tmp += _ref_list[i];
      
  _ref_list = tmp;

#if defined(DJV_DBG)
  dbg();
#endif
}

void Cache::purge()
{
  DJV_DBG_OBJ("cache::Cache::purge");
#if defined(DJV_DBG)
  //dbg();
#endif

  List<Ref *> tmp;
  tmp.resize(_ref_list.size());
  
  int size = 0;
  for (uint i = 0; i < _ref_list.size(); ++i)
    if (! _ref_list[i]->ref_count()) {
      _cache_size -= _ref_list[i]->get()->data_bytes();
      delete _ref_list[i];
    } else
      tmp[size++] = _ref_list[i];

  tmp.resize(size);
  _ref_list = tmp;
  
#if defined(DJV_DBG)
  //dbg();
#endif
}

void Cache::dbg()
{
  DJV_DBG_OBJ("cache::Cache::dbg");
  DJV_DBG_PRN("refs = " << _ref_list.size());
  DJV_DBG_PRN("cache max = " << uint(_cache_max / mem::megabyte));
  DJV_DBG_PRN("cache size = " << uint(_cache_size / mem::megabyte));
  
  for (uint i = 0; i < _ref_list.size(); ++i)
    DJV_DBG_PRN("item (ref = " << _ref_list[i]->ref_count() << ") = " <<
      _ref_list[i]->key() << " " << _ref_list[i]->frame());
}

const List<int> & size_default()
{
  static const List<int> data = List<int>() << 256 << 512 << 1024 << 2048 <<
    4096;
  return data;
}

const List<Str> & size_label()
{
  static const List<Str> data = List<Str>() << "256" << "512" << "1024" <<
    "2048" << "4096";
  return data;
}

namespace { std::auto_ptr<Cache> _cache; }

Cache * cache()
{
  if (! _cache.get()) _cache = std::auto_ptr<Cache>(new Cache);
  return _cache.get();
}

void del() { _cache.reset(); }

}

