/*------------------------------------------------------------------------------
 bin/djv_view/color_sample.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "color_sample.h"
#include <djv_ui/prefs.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/button.h>
#include <djv_ui/choice.h>
#include <djv_ui/num_int.h>
#include <djv_ui/color.h>
#include <djv_ui/app.h>
#include "prefs.h"
#include "view.h"

namespace color_sample {

Dialog::Dialog() :
  ui::dialog::Base("Color Sample"),
  _view(0),
  _value(image::pixel::RGBA_F32), _size(SIZE_3X3), _lock(false),
  _update(false)
{
  // Widgets.

  _widget = new ui::color::Widget;
  _swatch = new ui::color::Swatch;
  
  _size_widget = new ui::choice::Widget(size_label());
  ui::label::Widget * size_label = new ui::label::Widget("Size:");
  
  _lock_widget = new ui::button::Check("Pixel Lock");

  ui::button::Push * close_button = new ui::button::Push("Clos&e");
  
  // Layout.
  
  ui::layout::Vertical * layout = new ui::layout::Vertical(this);
  
  ui::layout::Horizontal * h_layout = new ui::layout::Horizontal(layout);
  h_layout->margin(0);
  h_layout->add(_swatch);
  h_layout->add(_widget);
  h_layout->stretch(_widget);
  
  layout->add_spacer(-1, true);
  
  h_layout = new ui::layout::Horizontal(layout);
  h_layout->margin(0);
  h_layout->add(size_label);
  h_layout->add(_size_widget);
  h_layout->add(_lock_widget);
  h_layout->add_spacer(-1, true);
  h_layout->add(close_button);
  
  // Initialize.
  
  ui::prefs::Prefs prefs(ui::prefs::app(), "color_sample");
  List<Str> tmp;
  image::pixel::PIXEL pixel = _value.pixel();
  if (prefs.get("size", &tmp))
    str::serialize(&tmp, &_size, color_sample::size_label());
  if (prefs.get("pixel", &tmp))
    str::serialize(&tmp, &pixel, image::pixel::pixel_label());
  _value.set(pixel);
  if (prefs.get("lock", &tmp)) str::serialize(&tmp, &_lock);

  widget_update();
  
  size(size_hint());
  close_button->take_focus();
  
  // Callbacks.

  _widget->sig.set(this, widget_callback);
  _size_widget->sig.set(this, size_callback);
  _lock_widget->sig.set(this, lock_callback);
  close_button->sig.set(this, close_callback);
}

void Dialog::widget_del()
{
  ui::prefs::Prefs prefs(ui::prefs::app(), "color_sample");
  prefs.set("size", str::serialize(_size, size_label()));
  prefs.set("pixel",
    str::serialize(_value.pixel(), image::pixel::pixel_label()));
  prefs.set("lock", str::serialize(_lock));

  ui::dialog::Base::widget_del();
}

void Dialog::set(const view::Widget * in, const Vec2i & pick)
{
  if (in == _view) return;
  _view = in;
  _pick = pick;
  if (shown() && visible()) widget_update();
  else _update = true;
}

void Dialog::update(const view::Widget * in, const Vec2i & pick)
{
  if (in != _view || ! shown() || ! visible()) return;
  
  DJV_DBG_OBJ("color_sample::Dialog::update");
  DJV_DBG_PRN("pick = " << pick);

  _pick = Vec2i(
    math::floor(pick.x / in->zoom()),
    math::floor(pick.y / in->zoom())
  );
  DJV_DBG_PRN("pick = " << _pick);
  
  widget_update();
}

void Dialog::del(const view::Widget * in)
{
  if (in != _view) return;
  _view = 0;
}

void Dialog::show()
{
  if (_update) widget_update();
  ui::dialog::Base::show();
}

void Dialog::widget_callback(const image::Color & in)
{
  _value = in;
  _swatch->set(in);
}

void Dialog::size_callback(int in)
{
  _size = SIZE(in);
  widget_update();
}

void Dialog::lock_callback(bool in)
{
  _lock = in;
  widget_update();
}

void Dialog::close_callback(bool)
{
  hide();
}

void Dialog::widget_update()
{
  if (! _view) return;
  const image::gl::data::Data * data = _view->get();
  if (! data) return;
  
  DJV_DBG_OBJ("color_sample::Dialog::widget_update");
  DJV_DBG_PRN("pick = " << _pick);
  DJV_DBG_PRN("lock = " << _lock);
  
  callbacks(false);

  if (! _lock && data->get()) _value.set(data->get()->pixel());
  else _value.set(_widget->get().pixel());
  
  // Render color sample.
  
  const int size = color_sample::size(_size);
  DJV_DBG_PRN("size = " << size);

  image::Data tmp(image::Info(Vec2i(size), _value.pixel()));

  image::gl::data::Options options;
  options.mirror_x = _view->mirror_x();
  options.mirror_y = _view->mirror_y();
  options.scale = ui::image::size(
    data->get() ? data->get()->size() : Vec2i(), _view->scale());
  options.rotate = ui::image::rotate(_view->rotate());
  options.profile = _view->profile();
  options.display = _view->display();
  options.channel = _view->channel();
  options.filter = _view->filter();
  options.background = _view->background();
  
  // XXX
  
  const int proxy_scale =
    data->get() ? djv::image::data::proxy_scale(data->get()->proxy()) : 1;
  DJV_DBG_PRN("proxy scale = " << proxy_scale);
  options.position = ui::image::position(-(_pick - Vec2i((size - 1) / 2)),
    options.scale * proxy_scale, _view->rotate());
  options.scale *= proxy_scale;
  DJV_DBG_PRN("options position = " << options.position);
  
  try { image::gl::data::copy(*data, _view->context(), &tmp, options); }
  catch (Error in) { ui::app::app()->error(in); }
  
  // Accumulate color sample.

  //mem::copy(tmp.data(), _value.data(), tmp.pixel_bytes());

  const int size2 = size * size;
  const int channels = image::pixel::channels(_value.pixel());
  switch (image::pixel::type(_value.pixel())) {

#define _WORK(TYPE) \
      const image::pixel::TYPE * p = \
        reinterpret_cast<image::pixel::TYPE *>(tmp.data()); \
      double accum [image::pixel::channels_max]; \
      for (int c = 0; c < channels; ++c) accum[c] = 0; \
      for (int i = 0; i < size2; ++i, p += channels) \
        for (int c = 0; c < channels; ++c) accum[c] += p[c] / double(size2);
    case image::pixel::U8: {
      _WORK(U8_T)
      for (int c = 0; c < channels; ++c) _value.set_u8(int(accum[c]), c);
    } break;
    case image::pixel::U16: {
      _WORK(U16_T)
      for (int c = 0; c < channels; ++c) _value.set_u16(int(accum[c]), c);
    } break;
    case image::pixel::F16: {
      _WORK(F16_T)
      for (int c = 0; c < channels; ++c) _value.set_f16(accum[c], c);
    } break;
    case image::pixel::F32: {
      _WORK(F32_T)
      for (int c = 0; c < channels; ++c) _value.set_f32(accum[c], c);
    } break;

    case image::pixel::U10: {
      const image::pixel::U10_S * p =
        reinterpret_cast<image::pixel::U10_S *>(tmp.data());
      double accum [image::pixel::channels_max];
      for (int c = 0; c < channels; ++c) accum[c] = 0;
      for (int i = 0; i < size2; ++i, ++p) {
        accum[0] += p->r / double(size2);
        accum[1] += p->g / double(size2);
        accum[2] += p->b / double(size2);
      }
      for (int c = 0; c < channels; ++c) _value.set_u10(int(accum[c]), c);
    } break;
  }
  DJV_DBG_PRN("value = " << _value);
  
  _widget->set(_value);
  _swatch->set(_value);
  _size_widget->set(_size);
  _lock_widget->set(_lock);
  
  callbacks(true);
}

int size(SIZE in)
{
  static const int data [] = { 1, 3, 5, 7, 9 };
  return data[in];
}

const List<Str> & size_label()
{
  static const List<Str> data = List<Str>() << "1x1" << "3x3" << "5x5" <<
    "7x7" << "9x9";
  return data;
}

namespace { std::auto_ptr<Dialog> _dialog; }

Dialog * dialog()
{
  if (! _dialog.get()) _dialog = std::auto_ptr<Dialog>(new Dialog);
  return _dialog.get();
}

void del()
{
  if (_dialog.get()) _dialog->widget_del();
  _dialog.release();
}

}

