/*------------------------------------------------------------------------------
 bin/djv_view/window_file.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "window.h"
#include <djv_ui/layout_row.h>
#include <djv_ui/button.h>
#include <djv_ui/menu.h>
#include <djv_ui/messages_dialog.h>
#include <djv_ui/prefs_dialog.h>
#include <djv_ui/about_dialog.h>
#include <djv_ui/file_browser.h>
#include <djv_ui/app.h>
#include "prefs.h"
#include <FL/Fl.H>

namespace window {

File_Group::File_Group() : _menu(0) {}

void File_Group::menu(ui::menu::Widget * in, int proxy, bool cache)
{
  in->menu_add("&File", 0, 0, 0, ui::menu::SUB_MENU);
  
  in->menu_add("&Open", FL_ALT + 'o', _open_callback, this);

  in->menu_add("Re&cent", 0, 0, 0, ui::menu::SUB_MENU);
  _menu_recent.clear();
  List<Str> tmp = prefs::file()->recent();
  for (uint i = 0; i < tmp.size(); ++i)
    _menu_recent += in->menu_add(ui::menu::list_name(tmp[i], i), 0,
      _recent_callback, this);
  in->menu_end();
  
  in->menu_add("&Reload", FL_ALT + 'r', _reload_callback, this);
  in->menu_add("Clos&e", FL_ALT + 'e', _close_callback, this,
    ui::menu::DIVIDER);

  in->menu_add("Prox&y", 0, 0, 0, ui::menu::SUB_MENU);
  _menu_proxy.clear();
  tmp = proxy_label();
  for (uint i = 0; i < tmp.size(); ++i)
    _menu_proxy += in->menu_add(tmp[i], '6' + i, _proxy_callback, this,
      ui::menu::RADIO | (int(i) == proxy ? ui::menu::VALUE : 0));
  in->menu_end();

  in->menu_add("&Cache", 0, _cache_callback, this,
    ui::menu::TOGGLE | (cache ? ui::menu::VALUE : 0) | ui::menu::DIVIDER);
  
  in->menu_add("&Messages", 0, _messages_callback, this);
  in->menu_add("&Preferences", 0, _prefs_callback, this);
  in->menu_add("&About", 0, _about_callback, this, ui::menu::DIVIDER);

  in->menu_add("Exit", FL_ALT + 'q', _exit_callback, this);
  
  in->menu_end();
  
  _menu = in;
}

void File_Group::toolbar(ui::layout::Horizontal * in)
{
  ui::button::Tool * open_widget = new ui::button::Tool("file_open");
  open_widget->tooltip("Open a file. (Shortcut: Alt + O)");
  
  ui::button::Tool * reload_widget = new ui::button::Tool("file_reload");
  reload_widget->tooltip("Reload the current file. (Shortcut: Alt + R)");
  
  ui::button::Tool * close_widget = new ui::button::Tool("file_close");
  close_widget->tooltip("Close the current file. (Shortcut: Alt + E)");
  
  in->add(open_widget);
  in->add(reload_widget);
  in->add(close_widget);
  
  open_widget->sig.set(this, open_callback);
  reload_widget->sig.set(this, reload_callback);
  close_widget->sig.set(this, close_callback);
}

void File_Group::open_callback(const File & in) { open_sig.emit(in); }
void File_Group::open_callback(bool)
  { ui::file_browser::set(this, open_callback); }
void File_Group::_open_callback() { open_callback(true); }

void File_Group::recent_callback(bool)
{
  File file = prefs::file()->recent()[list::find(_menu->menu(), _menu_recent)];
  if (file.seq_valid()) file.type = file::SEQ;
  open_sig.emit(file);
}

void File_Group::_recent_callback() { recent_callback(true); }

void File_Group::reload_callback(bool) { reload_sig.emit(true); }
void File_Group::_reload_callback() { reload_callback(true); }

void File_Group::close_callback(bool) { close_sig.emit(true); }
void File_Group::_close_callback() { close_callback(true); }

void File_Group::_proxy_callback()
  { proxy_sig.emit(list::find(_menu->menu(), _menu_proxy)); }

void File_Group::_cache_callback() { cache_sig.emit(_menu->value()); }

void File_Group::_messages_callback() { ui::messages_dialog::dialog()->show(); }
void File_Group::_prefs_callback() { ui::prefs_dialog::dialog()->show(); }
void File_Group::_about_callback() { ui::about_dialog::dialog()->show(); }
void File_Group::_exit_callback() { ui::app::app()->exit(); }

void Widget::file_open(const File in, bool init)
{
  DJV_DBG_OBJ("window::Widget::file_open");
  DJV_DBG_PRN("in = " << in);
  DJV_DBG_PRN("init = " << init);
  
  const bool valid = in.get().size() ? true : false;
  DJV_DBG_PRN("valid = " << valid);

  _load.reset();
  _info = Info();
  
  if (valid)
    try {
      _load = std::auto_ptr<image::io::Load>(image::io::load(in, &_info));
      _info.file = in;
      if (! init)
        prefs::file()->recent(_info.file);
    }
    catch (Error error) {
      ui::app::app()->error(str::Format("%1: %2").arg(error).arg(in.name()));
    }

  title((valid && _load.get()) ?
    str::Format("%1 - %2").arg(ui::app::app()->name()).arg(_info.file) :
    str::Format("%1").arg(ui::app::app()->name()));
  
  playback_time(! init ? _info.time : _playback_time);
  playback_frame(! init ? _info.time.start : _playback_frame);
  playback_update();
  image_update();
  file_cache_update();
  menu_update();

  if (! init) {
    if (prefs::window()->resize()) {
      if (! fullscreen()) window_fit(true);
      else _view->view_fit();
    }
    playback(
      (valid && (_info.time.start != _info.time.end) &&
        prefs::playback()->start()) ?
      playback::FORWARD : playback::STOP);
  }
}

void Widget::file_open_callback(const File & in) { file_open(in); }
void Widget::file_recent_callback(const List<Str> &) { menu_update(); }

void Widget::file_reload_callback(bool)
{
  DJV_DBG_OBJ("window::Widget::file_reload_callback");

  _load.reset();
  if (_info.file.get().size()) {
    try { _load = std::auto_ptr<image::io::Load>(
      image::io::load(_info.file, &_info)); }
    catch (Error error) {
      ui::app::app()->error(str::Format("%1: %2").arg(error).arg(_info.file));
    }
  }
  
  file_cache_update();
  playback_update();
  image_update();
  menu_update();
}

void Widget::file_reload_callback(image::io::Plugin * in)
{
  if (! _load.get()) return;
  
  if (! in) file_reload_callback(true);
  else if (in->name() == _load->name()) {
    const List<Str> & option_list = in->option_list();
    for (uint i = 0; i < option_list.size(); ++i) {
      List<Str> tmp = in->option(option_list[i]);
      _load->option(option_list[i], &tmp);
    }
    image_update();
  }
}

void Widget::file_close_callback(bool) { file_open(File()); }

void Widget::file_proxy_callback(int in)
{
  DJV_DBG_OBJ("window::Widget::file_proxy_callback");
  DJV_DBG_PRN("in = " << in);

  _file_proxy = in;
  file_cache_update();
  image_update();
  menu_update();
}

void Widget::file_cache_callback(bool in)
{
  DJV_DBG_OBJ("window::Widget::file_cache_callback");
  DJV_DBG_PRN("in = " << in);

  _file_cache = in;
  file_cache_update();
  menu_update();
}

void Widget::file_cache_update()
{
  DJV_DBG_OBJ("window::Widget::file_cache_update");
  
  callbacks(false);
  
  cache::cache()->del(int(this));
  
  callbacks(true);
}

}

