/*------------------------------------------------------------------------------
 bin/djv_vlut/app.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "app.h"
#include <djv_base/sys.h>
#include <djv_ui/prefs.h>
#include <djv_ui/prefs_dialog.h>

App::App(int & argc, char ** argv) :
  ui::app::App("djv_vlut", argc, argv),
  _input(sys::env("DISPLAY")),
  _window(0)
{
  DJV_DBG_OBJ("App::App");
  
  // Command line.
  
  bool r = false;
  try { r = cmdln(argc, argv); }
  catch (Error in) { error(str::Format("Command Line: %1").arg(in)); }
  if (! r) { djv::app::App::exit(1); return; }
  
  // Initialize.
  
  valid(true);
  
  ui::prefs::Prefs * prefs = ui::prefs::app();
  List<Str> tmp;
  if (prefs->get("value", &tmp)) {
    serialize(&tmp, &_color);
    _color.lut_load();
  }

  ui::prefs_dialog::dialog()->add("Image I/O", new ui::prefs_dialog::Image_Io);
  ui::prefs_dialog::dialog()->add("Global", new ui::prefs_dialog::Global);

  try {
    image::io::Info info;
    std::auto_ptr<image::io::Load> load(image::io::load(_input, &info, "vlut"));
    DJV_DBG_PRN("vlut = " << info);
    
    _image.set(image::Info(info.size, info.pixel));
    _image_tmp.set(image::Info(info.size, info.pixel));

    _save = std::auto_ptr<image::io::Save>(
      image::io::save(_input, info, "vlut"));
  }
  catch (Error in) { error(str::Format("%1: %2").arg(in).arg(_input)); }
  
  _window = new window::Widget;
  
  color_update();

  _window->show();
  
  // Callbacks.
  
  _window->sig.set(this, color_callback);
}

App::~App()
{
  DJV_DBG_OBJ("App::~App");
  
  if (_window) _window->widget_del();
  
  color_list_del();
  
  ui::prefs::Prefs * prefs = ui::prefs::app();
  prefs->set("value", serialize(_color));
}

void App::color_callback(const Color & in)
{
  _color = in;
  color_update();
}

void App::color_update()
{
  DJV_DBG_OBJ("App::color_update");
  DJV_DBG_PRN("color = " << _color);
  
  try {
    image::gl::data::gradient(&_image);
    image::gl::data::Options options;
    options.display = _color;
    image::gl::data::copy(_image, &_image_tmp, options);
    
    /*const image::pixel::F32_T * p =
      reinterpret_cast<const image::pixel::F32_T *>(_image_tmp.data());
    for (int i = 0; i < _image_tmp.size().x; ++i, ++p)
      DJV_DBG_PRN(i << " = " << *p);*/

    if (_save.get()) _save->set(_image_tmp);
  }
  catch (Error in) { error(in); }
  
  _window->set(_color);
}

bool App::cmdln(int argc, char ** argv) throw (Error)
{
  if (! app::App::cmdln(argc, argv)) return false;
  
  if (1 == argc) ;
  else if (2 == argc) _input = argv[1];
  else throw Error(argv[2]);
  
  return true;
}

Str App::help() const
{
  return str::Format(
"\n"
" djv_vlut\n"
" --------\n"
" Video lookup table utility.\n"
"\n"
" Usage\n"
" -----\n"
" djv_vlut [input] [option]...\n"
"\n"
" input - Input video lookup table. Default: %1.\n"
"\n"
"%2"
  ).
    arg(_input).
    arg(app::App::help());
}

int main(int argc, char ** argv)
{
  int r = 1;
  try { r = App(argc, argv).exec(); }
  catch (Error in) { error::print(in); }
  return r;
}

