/*------------------------------------------------------------------------------
 lib/djv_base/app.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_base/app.cxx"

#include "app.h"
#include <djv_base/math.h>
#include <djv_base/sys.h>
#include <djv_base/file.h>
#include <sstream>
#include <stdio.h>

namespace djv {
namespace app {

namespace { App * _app = 0; }

App::App(const Str & name, int & argc, char ** argv) :
  _exit(0),
  _name(name), _argv0(argc ? argv[0] : ""),
  _endline(false), _separator(false)
{
  _app = this;
  
  DJV_DBG_OBJ2(_DBG, "App::App");
  DJV_DBG_PRN("name = " << name);
}

App::~App()
{
  DJV_DBG_OBJ2(_DBG, "App::~App");
}

void App::exit(int in)
{
  _exit = in;
}

bool App::cmdln(int & argc, char ** argv)
{
  for (int i = 1; i < argc; ++i)
    if (str::compare("-help", argv[i]) || str::compare("-h", argv[i])) {
      message(help());
      return false;
    }
  for (int i = 1; i < argc; ++i)
    if (str::compare("-info", argv[i])) {
      print(info());
      return false;
    }
    else if (str::compare("-about", argv[i])) {
      message("\n" + about() + "\n");
      return false;
    }
  return true;
}

Str App::help() const
{
  return str::Format(
"General Options\n"
"---------------\n"
"-help, -h - Display help message.\n"
"-info - Display information message.\n"
"-about - Display about message.\n"
"\n"
"Command line options are case-insensitive.\n"
  );
}

Str App::info() const
{
  return str::Format(
"Search Paths:\n%1"
  ).
    arg(str::join(sys::search_path(), ", "));
}

Str App::about() const
{
  return str::Format(
"DJV Imaging %1\n"
"http://djv.sourceforge.net\n"
"Made in Berkeley, CA, USA\n"
"Copyright (c) 2004-2007 Darby Johnston. All rights reserved.\n"
"\n"
"Redistribution and use in source and binary forms, with or without "
"modification, are permitted provided that the following conditions are met:\n"
"\n"
"* Redistributions of source code must retain the above copyright notice, "
"this list of conditions, and the following disclaimer.\n"
"* Redistributions in binary form must reproduce the above copyright notice, "
"this list of conditions, and the following disclaimer in the documentation "
"and/or other materials provided with the distribution.\n"
"* Neither the names of the copyright holders nor the names of any "
"contributors may be used to endorse or promote products derived from this "
"software without specific prior written permission.\n"
"\n"
"THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "
"\"AS IS\" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED "
"TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR "
"PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR "
"CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, "
"EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, "
"PROCUREMENT OF SUBSTITUE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; "
"OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, "
"WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR "
"OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF "
"ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.\n"
"\n"
"This software is based in part on the works of:\n"
"\n"
"* FLTK - http://www.fltk.org, Copyright 1998-2006, Bill Spitzak.\n"
"* OpenEXR - http:://www.openexr.com, Copyright (c) 2004, Industrial Light & "
"Magic, a division of Lucasfilm Entertainment Company Ltd.\n"
"* GLEW - http://glew.sourceforge.net, Copyright (C) 2002-2007, Milan Ikits, "
"<milan ikits[]ieee org> Copyright (C) 2002-2007, Marcelo E. Magallon, "
"<mmagallo[]debian org> Copyright (C) 2002, Lev Povalahev, All rights "
"reserved.\n"
"* libjpeg - http://www.ijg.org, Copyright (c) 1991-1998, Thomas G. Lane.\n"
"* libtiff - http://www.libtiff.org, Copyright (c) 1988-1997 Sam Leffler, "
"  Copyright (c) 1991-1997 Silicon Graphics, Inc.\n"
"* libpng - http://www.libpng.org, Copyright (c) 2004, 2006 Glenn "
"Randers-Pehrson.\n"
"* libquicktime - http://libquicktime.sourceforge.net, Copyright (c) 2002 "
"Heroine Virtual Ltd., Copyright (c) 2002-2007 Members of the libquicktime "
"project, LGPL License http://www.gnu.org/licenses/lgpl.html."
  ).
    arg(DJV_VERSION);
}

void App::message(const Str & in) const
{
  const List<Str> tmp = str::format(in, sys::terminal_width() - 2);
  for (uint i = 0; i < tmp.size(); ++i)
    print(" " + tmp[i] + " ");
}

void App::error(const Str & in) const { error(Error(in)); }

void App::error(const Error & in) const
{
  if (_endline) {
    ::printf("\n");
    const_cast<App *>(this)->_endline = false;
  }
  print(error::format(in));
  //separator();
}

void App::print(const Str & in, bool newline) const
{
  if (_separator) {
    const_cast<App *>(this)->_separator = false;
    ::printf("\n");
  }
  
  if (newline) {
    ::printf("%s\n", in.c_str());
  } else {
    ::printf("%s", in.c_str());
    ::fflush(stdout);
  }

  const_cast<App *>(this)->_endline = ! newline;
}

void App::separator() const
{
  const_cast<App *>(this)->_separator = true;
}

App * app() { return _app; }

}
}

