/*------------------------------------------------------------------------------
 lib/djv_base/file.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_base/file.cxx"

#include "file.h"
#include <djv_base/math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <errno.h>

namespace djv {
namespace file {

File::File() :
  type(FILE), size(0), user(0), perm(0), time(0)
{}

File::File(const Str & in, bool stat) :
  type(FILE), size(0), user(0), perm(0), time(0)
{
  set(in, stat);
}

void File::set(const Str & in, bool stat)
{
  DJV_DBG_OBJ2(_DBG, "File::set");
  DJV_DBG_PRN("in = " << in);
  DJV_DBG_PRN("stat = " << stat);
  
  // Split file name into pieces.
  
  _path.clear();
  _base.clear();
  _number.clear();
  _extension.clear();
  split(in, &_path, &_base, &_number, &_extension);
  DJV_DBG_PRN("path = " << _path);
  DJV_DBG_PRN("base = " << _base);
  DJV_DBG_PRN("number = " << _number);
  DJV_DBG_PRN("extension = " << _extension);
  
  _name = _base + _number + _extension;
  
  // File information.
  
  type = FILE;
  size = 0;
  user = 0;
  perm = 0;
  time = 0;
  if (stat) this->stat();
  
  // Set file sequence.
  
  seq = str_to_seq(_number);
  DJV_DBG_PRN("seq = " << seq);
}

Str File::get(int frame, bool path) const
{
  Str out;
  
  // Reconstruct file name from pieces.
  
  if (path) out += _path;
  out += _base;
  out += SEQ == type ? (frame != -1 ? frame_to_str(frame, seq.pad) :
    seq_to_str(seq)) : _number;
  out += _extension;
  
  return out;
}

void File::path(const Str & in) { _path = in; }
void File::base(const Str & in) { _base = in; }
void File::number(const Str & in) { _number = in; }
void File::extension(const Str & in) { _extension = in; }

#if defined(WIN32)
#define _STAT ::_stati64
#define _STAT_FNC ::_stati64
#elif defined(DJV_OSX)
#define _STAT ::stat
#define _STAT_FNC ::stat
#else
#define _STAT ::stat64
#define _STAT_FNC ::stat64
#endif

bool File::stat(const Str & path)
{
  DJV_DBG_OBJ2(_DBG, "File::stat");
  DJV_DBG_PRN("path = " << path);

  // Stat file information.
  
  struct _STAT stat_buf;
  mem::set(0, &stat_buf, sizeof(struct stat));
  const Str file = path.size() ?
    (path + path_separator + get(-1, false)) : get();
  DJV_DBG_PRN("file = " << file);
  if (_STAT_FNC(file.c_str(), &stat_buf) != 0) {
    const Str err(strerror(errno));
    DJV_DBG_PRN("errno = " << err);
    return false;
  }
  
  size = stat_buf.st_size;
  user = stat_buf.st_uid;
  perm = 0;
  time = stat_buf.st_mtime;
  
#if defined(WIN32)
  if (stat_buf.st_mode & _S_IFDIR) type = DIR;
  perm |= (stat_buf.st_mode & _S_IREAD) ? READ : 0;
  perm |= (stat_buf.st_mode & _S_IWRITE) ? WRITE : 0;
  perm |= (stat_buf.st_mode & _S_IEXEC) ? EXEC : 0;
#else
  if (S_ISDIR(stat_buf.st_mode)) type = DIR;
  if (stat_buf.st_mode & S_IRUSR) perm |= READ;
  if (stat_buf.st_mode & S_IWUSR) perm |= WRITE;
  if (stat_buf.st_mode & S_IXUSR) perm |= EXEC;
#endif
  
  return true;
}

}
}
