/*------------------------------------------------------------------------------
 lib/djv_base/file.h

 File System Information
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_FILE_H
#define DJV_FILE_H

#include <djv_base/base.h>

namespace djv {
namespace file {

// File sequence.
  
class Seq {
public:
  Seq() : start(0), end(0), pad(0) {}
  int start, end, pad;
};

// File information.

enum TYPE { FILE, SEQ, DIR };
enum PERM { READ = 1, WRITE = 2, EXEC = 4 };

class File {
public:
  
  File();
  File(const Str &, bool stat = true);

  // Value.
  
  void set(const Str &, bool stat = true);
  Str get(int frame = -1, bool path = true) const;
  
  // Components.

  void path(const Str &);
  void base(const Str &);
  void number(const Str &);
  void extension(const Str &);
  const Str & path() const { return _path; }
  const Str & name() const { return _name; }
  const Str & base() const { return _base; }
  const Str & number() const { return _number; }
  const Str & extension() const { return _extension; }

  // Information.

  bool stat(const Str & path = Str());
  
  TYPE type;
  uint64_t size;
  uid_t user;
  int perm;
  time_t time;

  inline bool seq_add(const File &);
  inline bool seq_valid() const;
  Seq seq;

  // Conversion.

  operator Str() const { return get(); }

private:

  static void split(
    const Str &,
    Str * path, Str * base, Str * number, Str * extension
  );
  
  Str _path, _name, _base, _number, _extension;
};
  
// Functions.
  
void seq_compress(List<File> *);

enum FILTER {FILTER_NONE = 0, FILTER_FILE = 1, FILTER_DIR = 2,
  FILTER_HIDDEN = 4 };
void filter(List<File> *, int filter,
  const List<Str> & glob_list = List<Str>());

enum SORT { SORT_NAME, SORT_TYPE, SORT_SIZE, SORT_USER, SORT_PERM, SORT_TIME };
void sort(List<File> *, SORT, bool reverse = false);
List<File> sort_dirs(const List<File> &);

// Paths.
  
Str path_absolute(const Str &);
bool is_path_absolute(const Str &);
Str path_fix(const Str &);
Str path_posix(const Str &);

static Str path_separator_list = "/\\";
static Str list_separator_list = ":;";
#if defined(WIN32)
static Str path_separator(1, path_separator_list[1]);
static Str list_separator(1, list_separator_list[1]);
#else
static Str path_separator(1, path_separator_list[0]);
static Str list_separator(1, list_separator_list[0]);
#endif

// Conversion.

Str seq_to_str(const Seq &);
Seq str_to_seq(const Str &);

Str frame_to_str(int, int pad = 0);
inline int str_to_frame(const Str &, int * pad = 0);

bool serialize(List<Str> *, File *);
List<Str> serialize(const File &);

// Comparison.

bool operator == (const File &, const File &);
bool operator != (const File &, const File &);

// Labels.

const List<Str> & type_label();
Str size_label(uint64_t);
const Str & perm_label(int);
const List<Str> & sort_label();

// Debugging.

Dbg & operator << (Dbg &, const Seq &);
Dbg & operator << (Dbg &, const File &);

}
}

#include <djv_base/file_.h>

#endif

