/*------------------------------------------------------------------------------
 lib/djv_base/vec.h
 
 Vectors

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_VEC_H
#define DJV_VEC_H

#include <djv_base/base.h>

namespace djv {
namespace vec {

// Base vector.

template<typename T, int D>
class Vec {
public:

  Vec() { zero(); }
  Vec(const Vec<T, D> & in) { set(in); }
  Vec(T in) { set(in); }
  virtual ~Vec() {}
  
  T e [D];
  
  inline void set(const Vec<T, D> &);
  inline void set(T);
  inline void zero();
  Vec<T, D> & operator = (const Vec<T, D> & in) { set(in); return *this; }
  
  inline Vec<T, D> & operator += (const Vec<T, D> &);
  inline Vec<T, D> & operator -= (const Vec<T, D> &);
  inline Vec<T, D> & operator *= (const Vec<T, D> &);
  inline Vec<T, D> & operator /= (const Vec<T, D> &);
  inline Vec<T, D> & operator += (T);
  inline Vec<T, D> & operator -= (T);
  inline Vec<T, D> & operator *= (T);
  inline Vec<T, D> & operator /= (T);
};

// Two-dimensional vectors.

template<typename T>
class Vec2 : public Vec<T, 2> {
public:

#define _VEC2_INIT \
  x(Vec<T, 2>::e[0]), \
  y(Vec<T, 2>::e[1])

  Vec2() : _VEC2_INIT {}
  Vec2(const Vec2<T> & in) : Vec<T, 2>(in), _VEC2_INIT {}
  Vec2(const Vec<T, 2> & in) : Vec<T, 2>(in), _VEC2_INIT {}
  Vec2(T _x, T _y) : _VEC2_INIT { x = _x; y = _y; }
  Vec2(T in) : Vec<T, 2>(in), _VEC2_INIT {}
  
  T & x, & y;
  
  Vec2<T> & operator = (const Vec2<T> & in) { set(in); return *this; }
};

// Three-dimensional vectors.

template<typename T>
class Vec3 : public Vec<T, 3> {
public:

#define _VEC3_INIT \
  x(Vec<T, 3>::e[0]), \
  y(Vec<T, 3>::e[1]), \
  z(Vec<T, 3>::e[2])

  Vec3() : _VEC3_INIT {}
  Vec3(const Vec3<T> & in) : Vec<T, 3>(in), _VEC3_INIT {}
  Vec3(const Vec<T, 3> & in) : Vec<T, 3>(in), _VEC3_INIT {}
  Vec3(T _x, T _y, T _z) : _VEC3_INIT { x = _x; y = _y; z = _z; }
  Vec3(T in) : Vec<T, 3>(in), _VEC3_INIT {}
  
  T & x, & y, & z;
  
  Vec3<T> & operator = (const Vec3<T> & in) { set(in); return *this; }
};

// Functions.

#if defined(min) // XXX WIN32.
#undef min
#endif
#if defined(max)
#undef max
#endif
template<typename T, int D>
inline Vec<T, D> min(const Vec<T, D> &, const Vec<T, D> &);
template<typename T, int D>
inline Vec<T, D> max(const Vec<T, D> &, const Vec<T, D> &);
template<typename T, int D>
inline Vec<T, D> clamp(const Vec<T, D> &, const Vec<T, D> &, const Vec<T, D> &);

template <typename T, int D>
inline Vec<T, D> normalize(const Vec<T, D> &);
template <typename T, int D>
inline double dot(const Vec<T, D> &, const Vec<T, D> &);
template <typename T, int D>
inline double length(const Vec<T, D> &);
template<typename T, int D>
inline bool size_valid(const Vec<T, D> &);

template<typename T>
inline double aspect(const Vec2<T> &);
template <typename T>
inline Vec2<T> swap(const Vec2<T> &);

// Operators.

template <typename T, int D>
inline Vec<T, D> operator + (const Vec<T, D> &, const Vec<T, D> &);
template <typename T, int D>
inline Vec<T, D> operator - (const Vec<T, D> &, const Vec<T, D> &);
template <typename T, int D>
inline Vec<T, D> operator * (const Vec<T, D> &, const Vec<T, D> &);
template <typename T, int D>
inline Vec<T, D> operator / (const Vec<T, D> &, const Vec<T, D> &);

template <typename T, int D>
inline Vec<T, D> operator + (const Vec<T, D> &, T);
template <typename T, int D>
inline Vec<T, D> operator - (const Vec<T, D> &, T);
template <typename T, int D>
inline Vec<T, D> operator * (const Vec<T, D> &, T);
template <typename T, int D>
inline Vec<T, D> operator / (const Vec<T, D> &, T);

template <typename T, int D>
inline Vec<T, D> operator - (const Vec<T, D> &);

// Comparison.

template <typename T, int D>
inline bool operator == (const Vec<T, D> &, const Vec<T, D> &);
template <typename T, int D>
inline bool operator != (const Vec<T, D> &, const Vec<T, D> &);

template <typename T, int D>
inline bool operator <  (const Vec<T, D> &, const Vec<T, D> &);
template <typename T, int D>
inline bool operator <= (const Vec<T, D> &, const Vec<T, D> &);
template <typename T, int D>
inline bool operator >  (const Vec<T, D> &, const Vec<T, D> &);
template <typename T, int D>
inline bool operator >= (const Vec<T, D> &, const Vec<T, D> &);

// Conversion.

template <int D>
inline Vec<int, D> to_int(const Vec<double, D> &);
template <int D>
inline Vec<double, D> to_float(const Vec<int, D> &);

template <int D>
inline Vec<int, D> ceil(const Vec<double, D> &);
template <int D>
inline Vec<int, D> floor(const Vec<double, D> &);

template <typename T, int D>
inline bool serialize(List<Str> *, Vec<T, D> *);
template <typename T, int D>
inline List<Str> serialize(const Vec<T, D> &);

// Debugging.

template <typename T, int D>
inline Dbg & operator << (Dbg &, const Vec<T, D> &);

}
}

#include <djv_base/vec_.h>
#include <djv_base/vec_fnc_.h>

#endif

