/*------------------------------------------------------------------------------
 lib/djv_image/io.h
 
 Image I/O
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_IMAGE_IO_H
#define DJV_IMAGE_IO_H

#include <djv_base/file.h>
#include <djv_base/io.h>
#include <djv_base/plugin.h>
#include <djv_image/image.h>
#include <djv_image/time.h>

namespace djv {
namespace image {
namespace io {

// Image I/O information.

class Info : public image::Info {
public:
  
  Info() {}
  Info(const image::Info & in) : image::Info(in) {}
  
  time::Time time;
};

// Image I/O base classes.

class Plugin : public plugin::Plugin {
public:
  
  // Information.
  
  virtual const List<Str> & extension_list() const
    { return _null_extension_list; }
  virtual Str help() const { return Str(); }
  
  // Options.
  
  virtual bool option(const Str &, List<Str> *) { return false; }
  virtual List<Str> option(const Str &) const { return List<Str>(); }
  virtual const List<Str> & option_list() const { return _null_option_list; }
  
private:

  List<Str> _null_extension_list, _null_option_list;
};

class Load : public Plugin {
public:
  
  virtual void open(const File &, Info *) throw (Error) = 0;
  virtual const Image * get(int frame = -1, int proxy = 0) throw (Error) = 0;
  
protected:

  struct Cache {
    Cache() {}
    Cache(const File & file, int frame, int proxy) :
      file(file), frame(frame), proxy(proxy) {}
    File file;
    int frame;
    int proxy;
    bool operator == (const Cache & in) const
      { return in.file == file && in.frame == frame && in.proxy == proxy; }
  };
};

class Save : public Plugin {
public:
  
  virtual void open(const File &, const Info &) throw (Error) = 0;
  virtual void close() throw (Error) {}
  virtual void set(const Image &, int frame = -1) throw (Error) = 0;
};

// Image I/O factories.

class Load_Factory : public plugin::Factory {
public:

  Load_Factory();
  ~Load_Factory();
  
  bool valid(const File &, const Str & plugin = Str());
  Load * get(const File &, Info *, const Str & plugin = Str()) throw (Error);
  Load * copy(Load *) const;
  
  Str help() const;

  bool option(const Str & plugin, const Str &, List<Str> *);
  
  List<Load *> list() const;

private:

  virtual Str plugin_entry() const { return "djv_image_load"; }
};

class Save_Factory : public plugin::Factory {
public:

  Save_Factory();
  ~Save_Factory();
  
  bool valid(const File &, const Str & plugin = Str());
  Save * get(const File &, const Info &, const Str & plugin = Str())
    throw (Error);
  Save * copy(Save *) const;
  
  Str help() const;

  bool option(const Str & plugin, const Str &, List<Str> *);
  
  List<Save *> list() const;

private:

  virtual Str plugin_entry() const { return "djv_image_save"; }
};

// Labels.

static const Str
  error_namespace = "Image I/O",
  error_unrecognized = "Unrecognized File",
  error_unsupported = "Unsupported File";
using djv::io::error_open;
using djv::io::error_read;
using djv::io::error_write;

// Global.

bool valid(const File &, const Str & plugin = Str());

Load * load(const File &, Info * = 0, const Str & plugin = Str())
  throw (Error);
Save * save(const File &, const Info &, const Str & plugin = Str())
  throw (Error);

Load_Factory * load();
Save_Factory * save();

void init();
void del();

}
}
}

#endif

