/*------------------------------------------------------------------------------
 lib/djv_image/io_lut_save.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_image/io_lut_save.cxx"

#include "io_lut.h"

namespace djv {
namespace image {
namespace io {
namespace lut {

Str Save::help() const
{
  return str::Format(
"%1:\n"
"* spec (value) - File specification: %2. Default: %3.\n"
  ).
    arg(plugin_name).
    arg(str::join(spec_label(), ", ")).
    arg(spec_label()[_options.spec]);
}

Plugin * Save::copy() const
  { Save * out = new Save; out->_options = _options; return out; }

bool Save::option(const Str & in, List<Str> * data)
{
  if (str::compare(in, option_list()[OPTION_SPEC]))
    return str::serialize(data, &_options.spec, spec_label());
  return false;
}

List<Str> Save::option(const Str & in) const
{
  if (str::compare(in, option_list()[OPTION_SPEC]))
    return str::serialize(_options.spec, spec_label());
  return List<Str>();
}

const List<Str> & Save::option_list() const { return save_option_label(); }

void Save::open(const File & in, const Info & info) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Save::open");
  DJV_DBG_PRN("in = " << in);
  
  _file = in;
  
  // Temporary buffer.
  
  Info _info;
  _info.size = Vec2i(info.size.x, 1);
  
  pixel::TYPE type = pixel::type(info.pixel);
  switch (type) {
    case pixel::F16: 
    case pixel::F32: type = pixel::U16; break;
    default: break;
  }
  _info.pixel = pixel::pixel(pixel::format(info.pixel), type);
  
  _image.set(_info);
  DJV_DBG_PRN("image = " << _image);
}

namespace {

void _open_inferno(djv::Io * io, const data::Info & info) throw (Error)
{
  char tmp [str::cstr_size];
  int size = SNPRINTF(tmp, str::cstr_size, "LUT: %d %d\n\n",
    pixel::channels(info.pixel), info.size.x);
  io->set(tmp, size);
}

void _open_kodak(djv::Io *, const data::Info &) throw (Error) {}

}

void Save::_open(const File & in) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Save::_open");
  DJV_DBG_PRN("in = " << in);

  _io.open(in, djv::io::WRITE);
  
  _spec = _options.spec;
  if (SPEC_AUTO == _spec) {
    if (lut::extension_list[0] == in.extension()) _spec = SPEC_INFERNO;
    else _spec = SPEC_KODAK;
  }
  switch (_spec) {
    case SPEC_INFERNO: _open_inferno(&_io, _image.info()); break;
    case SPEC_KODAK: _open_kodak(&_io, _image.info()); break;
    default: break;
  }
}

void _write_inferno(djv::Io * io, const Image * out) throw (Error)
{
  List<Color> color;
  color.resize(out->w());
  for (int x = 0; x < out->w(); ++x) {
    color[x].set(out->pixel());
    mem::copy(out->data(x, 0), color[x].data(), pixel::bytes(out->pixel()));
  }
  
  for (int c = 0; c < pixel::channels(out->pixel()); ++c)
    for (int x = 0; x < out->w(); ++x) {
      int v = 0;
      switch (pixel::type(out->pixel())) {
        case pixel::U8: v = color[x].get_u8(c); break;
        case pixel::U10: v = color[x].get_u10(c); break;
        case pixel::U16: v = color[x].get_u16(c); break;
        default: break;
      }
      char tmp [str::cstr_size];
      int size = SNPRINTF(tmp, str::cstr_size, "%9d\n", v);
      io->set(tmp, size);
    }
}

void _write_kodak(djv::Io * io, const Image * out) throw (Error)
{
  for (int x = 0; x < out->w(); ++x) {
    Color color(out->pixel());
    mem::copy(out->data(x, 0), color.data(), pixel::bytes(out->pixel()));
    for (int c = 0; c < pixel::channels(out->pixel()); ++c) {
      int v = 0;
      switch (pixel::type(out->pixel())) {
        case pixel::U8: v = color.get_u8(c); break;
        case pixel::U10: v = color.get_u10(c); break;
        case pixel::U16: v = color.get_u16(c); break;
        default: break;
      }
      char tmp [str::cstr_size];
      int size = SNPRINTF(tmp, str::cstr_size, "%6d", v);
      io->set(tmp, size);
    }
    io->set_8('\n');
  }
}

void Save::set(const Image & in, int frame) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Save::set");
  DJV_DBG_PRN("in = " << in);
  DJV_DBG_PRN("frame = " << frame);
  
  // Open.

  _open(_file.get(frame));
  
  const Image * image_p = &in;
  if (in.info() != _image.info()) {
    DJV_DBG_PRN("convert = " << _image);
    _image.zero();
    gl::data::copy(in, &_image);
    image_p = &_image;
  }
  
  // Write.
  
  switch (_spec) {
    case SPEC_INFERNO: _write_inferno(&_io, image_p); break;
    case SPEC_KODAK: _write_kodak(&_io, image_p); break;
    default: break;
  }
  
  _io.close();
}

}
}
}
}
