/*------------------------------------------------------------------------------
 lib/djv_ui/color.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_ui/color.cxx"

#include "color.h"
#include <djv_ui/layout_row.h>
#include <djv_ui/choice.h>
#include <djv_ui/num_int.h>
#include <djv_ui/num_float.h>
#include <djv_ui/pixel.h>

namespace djv {
namespace ui {
namespace color {
using namespace djv::image::pixel;
using djv::image::Color;

Widget::Widget() :
  _value(RGB_U8),
  _float_min(0.0), _float_max(1.0)
{
  // Widgets.

  for (int i = 0; i < channels_max; ++i) {
    _int_widget[i] = new num::Int_Slider;
    _float_widget[i] = new num::Float_Slider;
  }
  
  _pixel_widget = new pixel::Widget;
  
  // Layout.

  size_policy(STRETCH, FIXED);
  
  List<layout::Item *> group;
  for (int i = 0; i < channels_max; ++i) {
    group += _int_widget[i];
    group += _float_widget[i];
  }
  layout::group(group);
  
  layout::Vertical * layout = new layout::Vertical(this);
  layout->margin(0);
  for (int i = 0; i < channels_max; ++i) {
    layout::Horizontal * h_layout = new layout::Horizontal(layout);
    h_layout->margin(0);
    h_layout->add(_int_widget[i]);
    h_layout->add(_float_widget[i]);
  }
  
  layout::Horizontal * h_layout = new layout::Horizontal(layout);
  h_layout->margin(0);
  h_layout->add_stretch();
  h_layout->add(_pixel_widget);
  
  // Initialize.
  
  widget_update();
  value_update();
  
  // Callbacks.

  for (int i = 0; i < channels_max; ++i) {
    _int_widget[i]->sig.set(this, int_callback);
    _float_widget[i]->sig.set(this, float_callback);
  }
  
  _pixel_widget->sig.set(this, pixel_callback);
}

void Widget::set(const Color & in)
{
  if (in == _value) return;
  
  DJV_DBG_OBJ2(_DBG, "Widget::set");
  DJV_DBG_PRN("in = " << in);
  
  const Color prev = _value;
  _value = in;
  if (_value.pixel() != prev.pixel()) widget_update();
  value_update();
  sig.emit(_value);
}

void Widget::float_range(double min, double max)
{
  if (min == _float_min && max == _float_max) return;
  _float_min = min;
  _float_max = max;
  set(_value);
  widget_update();
}

void Widget::int_callback(int)
{
  for (int i = 0; i < djv::image::pixel::channels(_value.pixel()); ++i)
    switch (djv::image::pixel::type(_value.pixel())) {
      case U8: _value.set_u8(_int_widget[i]->get(), i); break;
      case U10: _value.set_u10(_int_widget[i]->get(), i); break;
      case U16: _value.set_u16(_int_widget[i]->get(), i); break;
      case F16: _value.set_f16(_float_widget[i]->get(), i); break;
      case F32: _value.set_f32(_float_widget[i]->get(), i); break;
    }
  value_update();
  sig.emit(_value);
}

void Widget::float_callback(double) { int_callback(0); }

void Widget::pixel_callback(PIXEL in)
{
  Color tmp(in);
  djv::image::color::convert(_value, &tmp);
  set(tmp);
}

void Widget::widget_update()
{
  DJV_DBG_OBJ2(_DBG, "Widget::widget_update");
  DJV_DBG_PRN("value = " << _value);
  
  callbacks(false);
  
  const int channels = djv::image::pixel::channels(_value.pixel());
  
  static const Str tooltip0 [] = { "Luminance channel.", "Alpha channel." };
  static const Str tooltip1 [] = { "Red channel.", "Green channel.",
    "Blue channel.", "Alpha channel." };
  const Str * tooltip = 0;
  switch (djv::image::pixel::format(_value.pixel())) {
    case L:
    case LA: tooltip = tooltip0; break;
    case RGB:
    case RGBA: tooltip = tooltip1; break;
  }

  for (int i = 0; i < channels; ++i) {
    _int_widget[i]->range(0, djv::image::pixel::max(_value.pixel()));
    _int_widget[i]->tooltip(tooltip[i]);
    _float_widget[i]->range(_float_min, _float_max);
    _float_widget[i]->tooltip(tooltip[i]);
  }
  
  // Show/hide.
 
  for (int i = 0; i < channels_max; ++i)
    switch (djv::image::pixel::type(_value.pixel())) {
      case U8:
      case U10:
      case U16:
        if (i < channels) _int_widget[i]->show();
        else _int_widget[i]->hide();
        _float_widget[i]->hide();
        break;
      case F16:
      case F32:
        _int_widget[i]->hide();
        if (i < channels) _float_widget[i]->show();
        else _float_widget[i]->hide();
        break;
    }

  update();

  callbacks(true);
}

void Widget::value_update()
{
  DJV_DBG_OBJ2(_DBG, "Widget::value_update");
  
  callbacks(false);
  
  for (int i = 0; i < djv::image::pixel::channels(_value.pixel()); ++i)
    switch (djv::image::pixel::type(_value.pixel())) {
      case U8: _int_widget[i]->set(_value.get_u8(i)); break;
      case U10: _int_widget[i]->set(_value.get_u10(i)); break;
      case U16: _int_widget[i]->set(_value.get_u16(i)); break;
      case F16: _float_widget[i]->set(_value.get_f16(i)); break;
      case F32: _float_widget[i]->set(_value.get_f32(i)); break;
    }
  
  _pixel_widget->set(_value.pixel());
  
  callbacks(true);
}

}
}
}
