/*------------------------------------------------------------------------------
 lib/djv_ui/group.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_ui/group.cxx"

#include "group.h"
#include <djv_ui/style.h>
#include <FL/Fl.H>
#include <FL/Fl_Group.H>
#include <FL/fl_draw.H>

namespace djv {
namespace ui {
namespace group {

Frame::Frame(const Fl_Boxtype frame) :
  _frame(frame)
{
  _edges[0] = _edges[1] = _edges[2] = _edges[3] = true;
}

void Frame::edges(bool left, bool right, bool top, bool bottom)
{
  _edges[0] = left;
  _edges[1] = right;
  _edges[2] = top;
  _edges[3] = bottom;
}

void Frame::frame(const Fl_Boxtype in)
{
  _frame = in;
}

int Frame::frame_size() const { return Fl::box_dx(_frame); }

Box2i Frame::frame_geom() const
{
  const Box2i geom = this->geom();
  const int margin = frame_size();
  return Box2i(
    geom.x + (_edges[0] ? margin : 0),
    geom.y + (_edges[2] ? margin : 0),
    geom.w - (_edges[0] ? margin : 0) - (_edges[1] ? margin : 0),
    geom.h - (_edges[2] ? margin : 0) - (_edges[3] ? margin : 0));
}

void Frame::draw()
{
  DJV_DBG_OBJ2(_DBG, "Frame::draw");
  
  Fl_Group::draw();
  widget::Base::draw();
  
  const Box2i geom = this->geom();
  const int margin = frame_size();
  DJV_DBG_PRN("geom = " << geom);
  DJV_DBG_PRN("margin = " << margin);
  
  const Box2i box(
    geom.x - (_edges[0] ? 0 : margin),
    geom.y - (_edges[2] ? 0 : margin),
    geom.w + (_edges[0] ? 0 : margin) + (_edges[1] ? 0 : margin),
    geom.h + (_edges[2] ? 0 : margin) + (_edges[3] ? 0 : margin));
  DJV_DBG_PRN("box = " << box);
  
  fl_push_clip(geom.x, geom.y, geom.w, geom.h);
  fl_draw_box(_frame, box.x, box.y, box.w, box.h, color());
  fl_pop_clip();
}

void Frame::dirty()
{
  widget::Widget::dirty();
  
  const int margin = frame_size();
  const Vec2i frame_size(
    (_edges[0] ? margin : 0) + (_edges[1] ? margin : 0),
    (_edges[2] ? margin : 0) + (_edges[3] ? margin : 0));
  _size_hint = (layout() ? layout()->size_hint() : 0) + frame_size;
}

void Frame::resize(int x, int y, int w, int h)
{
  Fl_Widget::resize(x, y, w, h);
  layout::Item::resize(x, y, w, h);
  
  if (layout()) {
    const Box2i geom = frame_geom();
    layout()->resize(geom.x, geom.y, geom.w, geom.h);
  }
}

Divider::Divider(int margin) :
  _margin(margin)
{}

Divider::~Divider() {}

void Divider::margin(int in)
{
  _margin = in;
}

Horizontal_Divider::Horizontal_Divider(int margin) :
  Divider(margin)
{
  size_policy(FIXED, STRETCH);
}

void Horizontal_Divider::draw()
{
  const Box2i geom = this->geom();
  const Box2i box(geom.x + geom.w / 2 - 1, geom.y, 2, geom.h);
  
  fl_push_clip(box.x, box.y, box.w, box.h);
  fl_draw_box(FL_THIN_DOWN_BOX, box.x, box.y - 1, box.w, box.h + 2,
    FL_BACKGROUND_COLOR);
  fl_pop_clip();
}

namespace {
Vec2i divider_size_hint(int margin) { return Vec2i(2, 0) + margin * 2; }
}

Vec2i Horizontal_Divider::size_hint() const
  { return divider_size_hint(margin()); }

Vertical_Divider::Vertical_Divider(int margin) :
  Divider(margin)
{
  size_policy(STRETCH, FIXED);
}

void Vertical_Divider::draw()
{
  const Box2i geom = this->geom();
  const Box2i box(geom.x, geom.y + geom.h / 2 - 1, geom.w, 2);

  fl_push_clip(box.x, box.y, box.w, box.h);
  fl_draw_box(FL_THIN_DOWN_BOX, box.x - 1, box.y, box.w + 2, box.h,
    FL_BACKGROUND_COLOR);
  fl_pop_clip();
}

Vec2i Vertical_Divider::size_hint() const
  { return swap(divider_size_hint(margin())); }

}
}
}
