/*------------------------------------------------------------------------------
 lib/djv_ui/label.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_ui/label.cxx"

#include "label.h"
#include <djv_ui/style.h>
#include <FL/Fl_Image.H>
#include <FL/fl_draw.H>

namespace djv {
namespace ui {
namespace label {

Widget::Widget(const Str & in)
{
  //text_font(style::style()->font_bold());

  size_policy(FIXED, FIXED);
  align_policy(LEFT, CENTER);
  
  set(in);
}

void Widget::set(const Str & in)
{
  if (in == _value) return;

  DJV_DBG_OBJ2(_DBG, "Widget::set");
  DJV_DBG_PRN("in = " << in);

  _value = in;
  update();
  redraw();
}

namespace {
inline int offset(layout::Item::ALIGN_POLICY in, int size, int size_hint)
{
  int out = 0;
  switch (in) {
    case layout::Item::LEFT: break;
    case layout::Item::CENTER: out += size / 2 - size_hint / 2; break;
    case layout::Item::RIGHT: break;
  }
  return out;
}
}

void Widget::draw()
{
  widget::Widget::draw();

  const Box2i geom = this->geom();
  
  fl_font(text_font(), style::style()->font_size());
  fl_color(text_color());
  fl_draw(_value.c_str(),
    geom.x + offset(align_policy()[0], geom.w, _size_hint.x),
    geom.y + offset(align_policy()[1], geom.h, _size_hint.y) +
      fl_height() - fl_descent());
}

void Widget::dirty()
{
  widget::Widget::dirty();
  
  fl_font(text_font(), style::style()->font_size());
  
  _size_hint = Vec2i(str_width(_value), fl_height());
}

Text::Text(const Str & in) :
  _word_size_max(0), _space_size(0), _text_height(0)
{
  align_policy(LEFT, CENTER);
  
  set(in);
}

void Text::set(const Str & in)
{
  if (in == _value) return;

  DJV_DBG_OBJ2(_DBG, "Text::set");
  DJV_DBG_PRN("in = " << in);

  _value = in;
  
  _word_list.clear();
  List<Str> tmp;
  if (_value.size())
    tmp = str::split(_value, "\n", true);
  for (uint i = 0; i < tmp.size(); ++i) {
    if (tmp[i].size())
      _word_list += str::split(tmp[i], " ", true);
    if (i < tmp.size() - 1)
      _word_list += "\n";
  }
  _word_size_list.resize(_word_list.size() + 1);
  
  update();
  redraw();
}

void Text::draw()
{
  widget::Widget::draw();

  const Box2i geom = this->geom();
  
  fl_font(text_font(), style::style()->font_size());
  fl_color(FL_FOREGROUND_COLOR);
  
  int x = 0, y = 0;
  for (uint i = 0; i < _word_list.size(); ++i) {
    if (_word_list[i] != "\n")
      fl_draw(_word_list[i].c_str(),
        geom.x + offset(align_policy()[0], geom.w, _size_hint.x) + x,
        geom.y + offset(align_policy()[1], geom.h, _size_hint.y) + y +
          fl_height() - fl_descent());
    x += _word_size_list[i];
    if ("\n" == _word_list[i] ||
      ((x + _word_size_list[i + 1]) > (geom.w - 1))
    ) {
      x = 0;
      y += _text_height;
    }
  }
}

void Text::dirty()
{
  DJV_DBG_OBJ2(_DBG, "Text::dirty");
  
  widget::Widget::dirty();
  
  fl_font(text_font(), style::style()->font_size());
  
  _word_size_max = 0;
  uint i = 0;
  for (; i < _word_list.size(); ++i) {
    _word_size_list[i] = _word_list[i] != "\n" ?
      str_width(_word_list[i] + " ") : 0;
    _word_size_max = math::max(_word_size_max, _word_size_list[i]);
  }
  _word_size_list[i] = 0;
  
  _space_size = str_width(" ");
  _text_height = fl_height();
}

void Text::resize(int x, int y, int w, int h)
{
  DJV_DBG_OBJ2(_DBG, "Text::resize");
  DJV_DBG_PRN("in = " << Box2i(x, y, w, h));

  widget::Widget::resize(x, y, w, h);
  
  Vec2i size;
  int _x = 0, _y = _text_height;
  for (uint i = 0; i < _word_list.size(); ++i) {
    _x += _word_size_list[i];
    if ("\n" == _word_list[i] ||
      ((_x + _word_size_list[i + 1]) > (w - 1))
    ) {
      size.x = math::max(size.x, _x);
      _x = 0;
      _y += _text_height;
    }
  }
  size.y = _y;

  if (size != _size_hint) {
    _size_hint = size;
    _size_min = Vec2i(
      math::max(_word_size_max, style::style()->text_size()), size.y);
    update();
  }
}

Image::Image(Fl_Image * in) :
  _value(in)
{}

void Image::set(Fl_Image * in)
{
  _value = in;
  update();
  redraw();
}

void Image::draw()
{
  widget::Widget::draw();
  
  const Box2i geom = this->geom();
  
  if (_value)
    _value->draw(
      geom.x + (geom.w - _value->w()) / 2,
      geom.y + (geom.h - _value->h()) / 2);
}

Vec2i Image::size_hint() const
  { return _value ? Vec2i(_value->w(), _value->h()) : Vec2i(); }

}
}
}
