/*------------------------------------------------------------------------------
 plugin/djv_exr/load.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "plugin/djv_exr/load.cxx"

#include "plugin.h"
#include <djv_base/box.h>
#include <djv_image/time.h>
#include <ImfChannelList.h>
#include <ImfStandardAttributes.h>
#include <ImfHeader.h>
#include <ImfInputFile.h>

using namespace image;

extern "C" {
DJV_PLUGIN_EXPORT plugin::Plugin * djv_image_load() { return new Load; }
}

Load::Load() : _f(0) {}
Load::~Load() { _close(); }

Str Load::help() const
{
  return str::Format(
"%1:\n"
"* color (value) - Input color space: %2. Default: %3.\n"
"* exposure (value) (defog) (knee low) (knee high) (gamma) - Exposure values. "
"Default: %4 %5 %6 %7 %8.\n"
  ).
    arg(plugin_name).
    arg(str::join(color_label(), ", ")).
    arg(color_label()[_options.color]).
    arg(_options.exposure.value).
    arg(_options.exposure.defog).
    arg(_options.exposure.knee_low).
    arg(_options.exposure.knee_high).
    arg(_options.exposure.gamma);
}

plugin::Plugin * Load::copy() const
  { Load * out = new Load; out->_options = _options; return out; }

bool Load::option(const Str & in, List<Str> * data)
{
  if (str::compare(in, option_list()[OPTION_COLOR]))
    return str::serialize(data, &_options.color, color_label());
  else if (str::compare(in, option_list()[OPTION_EXPOSURE]))
    return serialize(data, &_options.exposure);
  return false;
}

List<Str> Load::option(const Str & in) const
{
  if (str::compare(in, option_list()[OPTION_COLOR]))
    return str::serialize(_options.color, color_label());
  else if (str::compare(in, option_list()[OPTION_EXPOSURE]))
    return serialize(_options.exposure);
  return List<Str>();
}

const List<Str> & Load::option_list() const { return load_option_label(); }

void Load::open(const File & in, image::io::Info * info) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Load::open");
  DJV_DBG_PRN("in = " << in);

  _file = in;
  _open(_file.get(_file.seq.start), info);
  
  if (file::SEQ == _file.type) {
    info->time.start = _file.seq.start;
    info->time.end = _file.seq.end;
  }
}

void Load::_open(const File & in, image::io::Info * info) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Load::_open");
  DJV_DBG_PRN("in = " << in);
  
  _close();
  
try {
  
  _f = new Imf::InputFile(in.get().c_str());
  
  // Information.
  
  info->size = ::box(_f->header().dataWindow()).size;
  info->mirror_y = true;

  const Imf::ChannelList & channel_list = _f->header().channels();
  const Imf::Channel * channel_r = channel_list.findChannel("R");
  const Imf::Channel * channel_g = channel_list.findChannel("G");
  const Imf::Channel * channel_b = channel_list.findChannel("B");
  const Imf::Channel * channel_a = channel_list.findChannel("A");
  const Imf::Channel * channel_l = channel_list.findChannel("Y");
  
  pixel::FORMAT format = pixel::FORMAT(0);
  pixel::TYPE type = pixel::TYPE(0);

  for (Imf::ChannelList::ConstIterator i = channel_list.begin(); i !=
    channel_list.end(); ++i
  ) {
    _channel_list += i.name();
    
    type = math::max(Imf::HALF == i.channel().type ? pixel::F16 : pixel::F32,
      type);
  }
  
  if (channel_r || channel_g || channel_b) {

    format = pixel::RGB;
    _channel_list = List<Str>() << "R" << "G" << "B";
  
    if (channel_a) {
      format = pixel::RGBA;
      _channel_list += "A";
    }

  } else if (channel_l) {
  
    format = pixel::L;
    _channel_list = List<Str>() << "Y";
  
    if (channel_a) {
      format = pixel::LA;
      _channel_list += "A";
    }
  
  } else {
  
    switch (_channel_list.size()) {
      case 1: format = pixel::L; break;
      case 2: format = pixel::LA; break;
      case 3: format = pixel::RGB; break;
      case 4:
      default: format = pixel::RGBA; break;
    }
  }
  
  if (! pixel::pixel(format, type, &info->pixel))
    throw Error(plugin_name, image::io::error_unsupported);

  DJV_DBG_PRN("pixel = " << info->pixel);
  DJV_DBG_PRN("channels = " << _channel_list);
  
  // Image tags.
  
  if (Imf::hasOwner(_f->header()))
    info->tag[tag::label()[tag::CREATOR]] =
      Imf::ownerAttribute(_f->header()).value();
  if (Imf::hasComments(_f->header()))
    info->tag[tag::label()[tag::DESCRIPTION]] =
      Imf::commentsAttribute(_f->header()).value();
  if (Imf::hasCapDate(_f->header()))
    info->tag[tag::label()[tag::TIME]] =
      Imf::capDateAttribute(_f->header()).value();
  if (Imf::hasutcOffset(_f->header()))
    info->tag[tag::label()[tag::UTC_OFFSET]] =
      str::float_to_str(Imf::utcOffsetAttribute(_f->header()).value());
  
  if (Imf::hasLongitude(_f->header()))
    info->tag[tag_label()[TAG_LONGITUDE]] =
      str::float_to_str(Imf::longitudeAttribute(_f->header()).value());
  if (Imf::hasLatitude(_f->header()))
    info->tag[tag_label()[TAG_LATITUDE]] =
      str::float_to_str(Imf::latitudeAttribute(_f->header()).value());
  if (Imf::hasAltitude(_f->header()))
    info->tag[tag_label()[TAG_ALTITUDE]] =
      str::float_to_str(Imf::altitudeAttribute(_f->header()).value());
  
  if (Imf::hasFocus(_f->header()))
    info->tag[tag_label()[TAG_FOCUS]] =
      str::float_to_str(Imf::focusAttribute(_f->header()).value());
  if (Imf::hasExpTime(_f->header()))
    info->tag[tag_label()[TAG_EXPOSURE]] =
      str::float_to_str(Imf::expTimeAttribute(_f->header()).value());
  if (Imf::hasAperture(_f->header()))
    info->tag[tag_label()[TAG_APERTURE]] =
      str::float_to_str(Imf::apertureAttribute(_f->header()).value());
  if (Imf::hasIsoSpeed(_f->header()))
    info->tag[tag_label()[TAG_ISO_SPEED]] =
      str::float_to_str(Imf::isoSpeedAttribute(_f->header()).value());

  if (Imf::hasChromaticities(_f->header())) {
    const Imf::Chromaticities data =
      Imf::chromaticitiesAttribute(_f->header()).value();
    info->tag[tag_label()[TAG_CHROMATICITIES]] = str::join(List<Str>() <<
      str::float_to_str(data.red.x) << str::float_to_str(data.red.y) <<
      str::float_to_str(data.green.x) << str::float_to_str(data.green.y) <<
      str::float_to_str(data.blue.x) << str::float_to_str(data.blue.y) <<
      str::float_to_str(data.white.x) << str::float_to_str(data.white.y), " ");
  }
  if (Imf::hasWhiteLuminance(_f->header()))
    info->tag[tag_label()[TAG_WHITE_LUMINANCE]] =
      str::float_to_str(Imf::whiteLuminanceAttribute(_f->header()).value());
  if (Imf::hasXDensity(_f->header()))
    info->tag[tag_label()[TAG_X_DENSITY]] =
      str::float_to_str(Imf::xDensityAttribute(_f->header()).value());
  
  if (Imf::hasKeyCode(_f->header())) {
    const Imf::KeyCode data = Imf::keyCodeAttribute(_f->header()).value();
    info->tag[tag::label()[tag::KEYCODE]] = image::time::keycode_to_str(
      data.filmMfcCode(), data.filmType(), data.prefix(), data.count(),
      data.perfOffset());
  }
  if (Imf::hasTimeCode(_f->header()))
    info->tag[tag::label()[tag::TIMECODE]] = image::time::timecode_to_str(
      Imf::timeCodeAttribute(_f->header()).value().timeAndFlags());

  DJV_DBG_PRN("info = " << *info);

} catch (const std::exception & error) {
  throw Error(plugin_name, error.what());
}
}

void Load::_close()
{
  if (_f) { delete _f; _f = 0; }
}

const Image * Load::get(int frame, int proxy) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Load::get");
  DJV_DBG_PRN("frame = " << frame);
  DJV_DBG_PRN("proxy = " << proxy);
  DJV_DBG_PRN("this = " << int(this));
    
  profile::Profile profile;
  if (COLOR_EXPOSURE == _options.color) {
    DJV_DBG_PRN("profile");
    profile.type = image::profile::EXPOSURE;
    profile.exposure = _options.exposure;
  }
  
  // Open.
  
  const File file = File(_file.get(frame != -1 ? frame : _file.seq.start));
  DJV_DBG_PRN("file = " << file);
  
  const Cache cache(file, frame, proxy);
  if (cache == _cache) {
    DJV_DBG_PRN("cached");
    _image.profile(profile);
    return &_image;
  }
  _cache = cache;

  image::io::Info info;
  info.profile = profile;
  _open(file, &info);
  
  // Read.
  
  Image * data = proxy ? &_image_tmp : &_image;
  data->set(info);

  const bool mirror = Imf::DECREASING_Y == _f->header().lineOrder();
  const int channels = pixel::channels(info.pixel);
  const int bytes = pixel::channel_bytes(info.pixel);
  DJV_DBG_PRN("mirror = " << mirror);
  DJV_DBG_PRN("channels = " << channels);
  DJV_DBG_PRN("bytes = " << bytes);
  
try {
  const Vec2i file_offset(_f->header().dataWindow().min.x,
    _f->header().dataWindow().min.y);
  DJV_DBG_PRN("data window = " << file_offset);

  Imf::FrameBuffer frame_buffer;
  for (int c = 0; c < channels; ++c)
    frame_buffer.insert(_channel_list[c].c_str(),
      Imf::Slice(
        pixel::type(data->pixel()) == pixel::F16 ? Imf::HALF : Imf::FLOAT,
        (char *)data->data() -
          (file_offset.y * info.size.x * channels * bytes) -
          (file_offset.x * channels * bytes) +
          c * bytes,
        channels * bytes, info.size.x * channels * bytes, 1, 1,
        0.0));
  _f->setFrameBuffer(frame_buffer);

  if (mirror)
    for (int y = 0; y < info.size.y; ++y)
      _f->readPixels(file_offset.y + (info.size.y - 1 - y));
  else
    _f->readPixels(file_offset.y, file_offset.y + info.size.y - 1);

} catch (const std::exception & error) {
  throw Error(plugin_name, error.what());
}

  if (proxy) {
    info.size = data::proxy_scale(info.size, proxy);
    info.proxy = proxy;
    _image.set(info);
    data::proxy_scale(_image_tmp, &_image, proxy);
  }

  DJV_DBG_PRN("image = " << _image);
  return &_image;
}

