/*------------------------------------------------------------------------------
 plugin/djv_png/load.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "plugin/djv_png/load.cxx"

#include "plugin.h"

using namespace image;

extern "C" {
DJV_PLUGIN_EXPORT plugin::Plugin * djv_image_load() { return new Load; }
}

Load::Load() : _f(0), _png(0), _png_info(0), _png_info_end(0) {}
Load::~Load() { _close(); }

void Load::open(const File & in, image::io::Info * info) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Load::open");
  DJV_DBG_PRN("in = " << in);

  _file = in;
  _open(_file.get(_file.seq.start), info);

  if (file::SEQ == _file.type) {
    info->time.start = _file.seq.start;
    info->time.end = _file.seq.end;
  }
}

namespace {

bool _png_open(FILE * f, png_structp png, png_infop * png_info,
  png_infop * png_info_end)
{
  if (setjmp(png_jmpbuf(png))) return false;
  
  *png_info = png_create_info_struct(png);
  if (! *png_info) return false;
  *png_info_end = png_create_info_struct(png);
  if (! *png_info_end) return false;
  
  uint8_t tmp [8];
  if (fread(tmp, 8, 1, f) != 1) return false;
  if (png_sig_cmp(tmp, 0, 8)) return false;
  png_init_io(png, f);
  png_set_sig_bytes(png, 8);
  png_read_info(png, *png_info);
  
  if (png_get_interlace_type(png, *png_info) != PNG_INTERLACE_NONE)
    return false;
  png_set_expand(png);
  png_set_gray_1_2_4_to_8(png);
  png_set_palette_to_rgb(png);
  png_set_tRNS_to_alpha(png);
    
  return true;
}

}

void Load::_open(const File & in, image::io::Info * info) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Load::_open");
  DJV_DBG_PRN("in = " << in);
  
  _close();

  // Initialize libpng.
  
  _png = png_create_read_struct(PNG_LIBPNG_VER_STRING, 0, djv_png_error,
    djv_png_warning);
  if (! _png)
    throw Error(plugin_name, _png_error.msg);
  _png->error_ptr = &_png_error;
  
  // Open.
  
  _f = fopen(in.get().c_str(), "rb");
  if (! _f)
    throw Error(plugin_name, image::io::error_open);
  if (! _png_open(_f, _png, &_png_info, &_png_info_end))
    throw Error(plugin_name, _png_error.msg);
  
  // Information.
  
  info->size = Vec2i(png_get_image_width(_png, _png_info),
    png_get_image_height(_png, _png_info));
  
  int channels = png_get_channels(_png, _png_info);
  if (png_get_color_type(_png, _png_info) == PNG_COLOR_TYPE_PALETTE)
    channels = 3;
  if (png_get_valid(_png, _png_info, PNG_INFO_tRNS))
    ++channels;
  DJV_DBG_PRN("channels = " << channels);
  
  int bit_depth = png_get_bit_depth(_png, _png_info);
  if (bit_depth < 8) bit_depth = 8;
  DJV_DBG_PRN("bit depth = " << bit_depth);
  
  if (! pixel::pixel(channels, bit_depth, &info->pixel))
    throw Error(plugin_name, image::io::error_unsupported);
}

void Load::_close()
{
  if (_png || _png_info || _png_info_end) {
    png_destroy_read_struct(
      _png ? &_png : 0,
      _png_info ? &_png_info : 0,
      _png_info_end ? &_png_info_end : 0
    );
    _png = 0;
    _png_info = 0;
    _png_info_end = 0;
  }
  if (_f) { ::fclose(_f); _f = 0; }
}

namespace {

bool _scanline(png_structp png, uint8_t * out)
{
  if (setjmp(png_jmpbuf(png))) return false;
  png_read_row(png, out, 0);
  return true;
}

bool _end(png_structp png, png_infop png_info)
{
  if (setjmp(png_jmpbuf(png))) return false;
  png_read_end(png, png_info);
  return true;
}

}

const Image * Load::get(int frame, int proxy) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Load::get");
  DJV_DBG_PRN("frame = " << frame);
  DJV_DBG_PRN("proxy = " << proxy);
  
  // Open.
  
  image::io::Info info;
  _open(_file.get(frame != -1 ? frame : _file.seq.start), &info);
  
  // Read.
  
  Image * data = proxy ? &_image_tmp : &_image;
  data->set(info);
  
  for (int y = 0; y < info.size.y; ++y)
    if (! _scanline(_png, data->data(0,  data->h() - 1 - y)))
      throw Error(plugin_name, _png_error.msg);
  _end(_png, _png_info_end);
  
  if (proxy) {
    info.size = data::proxy_scale(info.size, proxy);
    info.proxy = proxy;
    _image.set(info);
    data::proxy_scale(_image_tmp, &_image, proxy);
  }
  
  DJV_DBG_PRN("image = " << _image);
  return &_image;
}

