/*------------------------------------------------------------------------------
 plugin/djv_quicktime_unix/load.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "plugin/djv_quicktime_unix/load.cxx"

#include "plugin.h"

using namespace image;

extern "C" {
DJV_PLUGIN_EXPORT plugin::Plugin * djv_image_load() { return new Load; }
}

Load::Load() : _frame(0), _f(0) {}
Load::~Load() { _close(); }

void Load::open(const File & in, image::io::Info * info) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Load::open");
  DJV_DBG_PRN("in = " << in);

  _close();
  
  // Open.
  
  _file = in;
  _f = lqt_open_read(_file.get().c_str());
  if (! _f)
    throw Error(plugin_name, image::io::error_open);
  
  if (! quicktime_has_video(_f))
    throw Error(plugin_name, error_video);

  if (! quicktime_supported_video(_f, 0))
    throw Error(plugin_name, image::io::error_unsupported);

  // Information.
  
  _info.size = Vec2i(quicktime_video_width(_f, 0),
    quicktime_video_height(_f, 0));
  
  static int cmodel_list [] = {
    BC_RGB888,
    BC_BGR888,
    BC_RGB161616,
    BC_RGBA8888,
    BC_RGBA16161616,
    LQT_COLORMODEL_NONE
  };
  const int cmodel = lqt_get_best_colormodel(_f, 0, cmodel_list);
  switch (cmodel) {
    case BC_BGR888: _info.bgr = true;
    case BC_RGB888: _info.pixel = pixel::RGB_U8; break;
    case BC_RGB161616: _info.pixel = pixel::RGB_U16; break;
    case BC_RGBA8888: _info.pixel = pixel::RGBA_U8; break;
    case BC_RGBA16161616: _info.pixel = pixel::RGBA_U16; break;
    default: throw Error(plugin_name, image::io::error_unsupported);
  }
  lqt_set_cmodel(_f, 0, cmodel);
  
  int time_constant = 0;
  const int time_scale = lqt_video_time_scale(_f, 0);
  const int frame_duration = lqt_frame_duration(_f, 0, &time_constant);
  _info.time = image::time::Time(
    0, frame_duration ? (lqt_video_duration(_f, 0) / frame_duration - 1) : 0,
    image::time::Speed(time_scale, frame_duration));
  DJV_DBG_PRN("time = " << _info.time);
  DJV_DBG_PRN("time constant = " << time_constant);
  
  const List<Str> & tag_label = tag::label();
  char * tag = lqt_get_artist(_f);
  if (tag) _info.tag[tag_label[tag::CREATOR]] = tag;
  tag = lqt_get_comment(_f);
  if (tag) _info.tag[tag_label[tag::DESCRIPTION]] = tag;
  tag = quicktime_get_copyright(_f);
  if (tag) _info.tag[tag_label[tag::COPYRIGHT]] = tag;
  
  *info = _info;
}

void Load::_close()
{
  DJV_DBG_OBJ2(_DBG, "Load::_close");
  
  if (_f) { quicktime_close(_f); _f = 0; }
}

const Image * Load::get(int frame, int proxy) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Load::get");
  DJV_DBG_PRN("frame = " << frame);
  DJV_DBG_PRN("proxy = " << proxy);
  
  if (-1 == frame) frame = 0;

  Image * data = proxy ? &_image_tmp : &_image;
  data->set(_info);
  
  struct Rows {
    Rows(Data * in) : p(0)
    {
      p = new unsigned char * [in->h() + 1];
      for (int y = 0; y < in->h(); ++y)
        p[y] = reinterpret_cast<pixel::U8_T *>(in->data(0, in->h() - 1 - y));
    }
    ~Rows() { if (p) delete [] p; }
    unsigned char ** p;
  } rows(data);

  // XXX
  
  if (frame != _frame + 1) {
    const uint64_t time = frame * _info.time.speed.duration;
    DJV_DBG_PRN("time = " << int(time));
    lqt_seek_video(_f, 0, time);
  }
  _frame = frame;
  
  lqt_decode_video(_f, rows.p, 0);
  
  if (proxy) {
    Info info = _info;
    info.size = data::proxy_scale(info.size, proxy);
    info.proxy = proxy;
    _image.set(info);
    data::proxy_scale(_image_tmp, &_image, proxy);
  }
  
  DJV_DBG_PRN("image = " << _image);
  return &_image;
}

